precision mediump float;

out lowp vec4		fragColor;
in highp vec2		texcoord;
in highp vec3		normal;

uniform bool		drawClouds;
uniform bool		drawDayNight;
uniform highp float	cloudsRot;
uniform highp vec3	lightv;
//uniform highp vec3	specularv;

uniform sampler2D	earthDayTex;
uniform sampler2D	earthNightTex;
uniform sampler2D	earthCloudsTex;

vec3 rgb2hsv(vec3 c)
{
	highp vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
	highp vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
	highp vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
	
	float d = q.x - min(q.w, q.y);
	float e = 1.0e-10;
	return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

void main()
{
	float M_PI = 3.14159265358979323846264;
	highp vec4 k = vec4(0,0,0,1);
	highp vec4 w = vec4(1,1,1,1);
	highp vec4 c = vec4(0,0,0,0);
	highp vec4 nightClouds = w;
	
	highp vec4 outColor;
	
	highp vec4 dayColor = texture(earthDayTex, texcoord);
	highp vec4 nightColor = texture(earthNightTex, texcoord);
	highp vec4 cloudsColor = texture(earthCloudsTex, texcoord);
	
	cloudsColor[1] = cloudsColor[0];
	cloudsColor[2] = cloudsColor[0];
	cloudsColor[3] = cloudsColor[0];
	
	outColor = mix(dayColor, nightColor, .1);
	
	//vec3 lightv = vec3(0,.6,-.8);
	
	float dnorm_f = dot(lightv, normal);
	
	dnorm_f = clamp(dnorm_f, 0.0, 1.0);
	
	// Rotate our clouds coordinate according to uniform cloudsRot
	highp vec2 texCoordClouds;
	texCoordClouds = texcoord;
	texCoordClouds[0] = texCoordClouds[0] + cloudsRot;
	
	highp vec4 texColor_day = texture(earthDayTex, texcoord);
	highp vec4 texColor_day_gamma;
	highp vec4 texColor_night;
	highp vec4 texColor_night_gamma;
	highp vec4 texColor_clouds;
	float reflectivity = 0.0;
	
	// Compute reflectivity
	highp vec3 rgb;
	rgb[0] = texColor_day[0];
	rgb[1] = texColor_day[1];
	rgb[2] = texColor_day[2];
	highp vec3 hsv = rgb2hsv(rgb);
	
	float valSmoothed0;
	float valSmoothed1;
	
	valSmoothed0 = smoothstep(.78, .82, hsv[2]);
	valSmoothed1 = 1. - smoothstep(.12, .15, hsv[2]);
	
	reflectivity = clamp(valSmoothed0 + valSmoothed1, 0.0, 1.0);
	
	
	// Brighten day texture
	//texColor_day_gamma = pow(texColor_day, vec4(.7,.7,.7,1));
	
	
	// More sophisticated curve function, for processing unmodified NASA images
	// y = ((x + x^0.5) / 2) ^ 0.8
	texColor_day_gamma = pow(((texColor_day + (pow(texColor_day, vec4(.5,.5,.5,1)))) * 0.5), vec4(0.8,0.8,0.8,1));
	
	
	// Compute light
	float light = dot(lightv, normal);
	float light_mix;
	//float specular;
	
	light_mix = smoothstep(-.3, .3, light);
	
	//specular = dot(specularv, normal);
	//specular = smoothstep(1.6, 2.0, specular) * reflectivity;
	
	if (drawDayNight) {
		nightClouds = vec4(.1,.1,.27,1);
		texColor_night = texture(earthNightTex, texcoord);
		//texColor_night_gamma = pow(((texColor_night + (pow(texColor_night, vec4(.5,.5,.5,1)))) * 0.5), vec4(0.8,0.8,0.8,1));
		
		
		// rough desaturate
		texColor_night.x *= 1.0;
		texColor_night.y *= 0.85;
		texColor_night.z *= 0.65;
		
		// levels
		texColor_night_gamma = clamp(texColor_night * 2.0, 0.0, 1.0);
		
		outColor = mix(texColor_night_gamma, texColor_day_gamma, light_mix);
	}
	//outColor = pow(outColor, vec4(1.0-(.84*specular),1.0-(.8*specular),1.0-(.7*specular),1.0));
	//outColor = clamp(outColor,0.0,1.0);
	
	if (drawClouds) {
		// Add clouds
		texColor_clouds = texture(earthCloudsTex, texCoordClouds);
		highp vec4 cloudColor = mix(nightClouds, w, light_mix);
		outColor = mix(outColor, cloudColor, texColor_clouds[0]);
	}
	
	fragColor = outColor;
}



